#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "console.h"
#include "settingsdialog.h"
#include <QLabel>
#include <QMessageBox>
#include "about.h"
#include <QTimer>
bool m_localEchoEnabled = false; // Add this line to MainWindow class
bool HEX = false;
MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    m_textEdit(new QTextEdit),
    m_console(new Console),
    m_status(new QLabel),
    m_ui(new Ui::MainWindow),
    m_settings(new SettingsDialog),
    m_serial(new QSerialPort(this)),
    m_about(new About),
    m_connectionTimer(new QTimer(this))  // Инициализируем таймер
{
    m_ui->setupUi(this);
    setWindowIcon(QIcon(":/images/deployer.png"));
    setWindowTitle("madCOM v1.4");
    setWindowFlags(windowFlags() ^ Qt::WindowContextHelpButtonHint);
    m_console->setEnabled(false);
    setCentralWidget(m_textEdit);
    m_ui->actionConnect->setEnabled(true);
    m_ui->actionDisconnect->setEnabled(false);
    m_ui->actionQuit->setEnabled(true);
    m_ui->actionConfigure->setEnabled(true);
    m_ui->statusBar->addWidget(m_status);
    initActionsConnections();
    connect(m_serial, &QSerialPort::errorOccurred, this, &MainWindow::handleError);
    connect(m_serial, &QSerialPort::readyRead, this, &MainWindow::readData);
    connect(m_textEdit, &QTextEdit::textChanged, this, &MainWindow::handleTextChanged);
    connect(m_settings, &SettingsDialog::hexStateChanged, this, &MainWindow::handleHexStateChanged);
    // Настраиваем таймер для управления таймаутом
    connect(m_connectionTimer, &QTimer::timeout, this, &MainWindow::handleConnectionTimeout);
}
void MainWindow::handleConnectionTimeout()
{
    // Если порт не открыт за отведенное время, закрываем его и выводим сообщение об ошибке
    if (!m_serial->isOpen()) {
        m_connectionTimer->stop();
        m_serial->close();
        QMessageBox::warning(this, tr("Timeout Error"), tr("Connection timed out."));
        showStatusMessage(tr("Connection timed out"));
        m_ui->actionConnect->setEnabled(true);
        m_ui->actionDisconnect->setEnabled(false);
    }
}
void MainWindow::handleHexStateChanged(bool state)
{
    HEX = state;
    m_ui->HEXmode->setChecked(state); // Предполагается, что у вас есть QAction actionHEX в ui
}
void MainWindow::initActionsConnections()
{
    connect(m_ui->actionConnect, &QAction::triggered, this, &MainWindow::openSerialPort);
    connect(m_ui->actionDisconnect, &QAction::triggered, this, &MainWindow::closeSerialPort);
    connect(m_ui->actionQuit, &QAction::triggered, this, &MainWindow::close);
    connect(m_ui->actionConfigure, &QAction::triggered, [this](){
        m_settings->show();
        m_settings->updateHexCheckbox();
    });
    connect(m_ui->actionClear, &QAction::triggered, m_console, &Console::clear);
    connect(m_ui->actionAbout, &QAction::triggered, this, &MainWindow::about);
}
void MainWindow::handleTextChanged()
{
    if (m_localEchoEnabled) {
        QString newText = m_textEdit->toPlainText();
        m_textEdit->clear();  // Очищаем текстовое поле
        m_textEdit->append(newText);  // Отображаем ввод пользователя
        // Выполняйте другие необходимые операции здесь, если нужно
    }
}
MainWindow::~MainWindow()
{
    delete m_settings;
    delete m_about;
    delete m_ui;
}
void MainWindow::openSerialPort()
{
    const SettingsDialog::Settings p = m_settings->settings();
    m_serial->setPortName(p.name);
    m_serial->setBaudRate(p.baudRate);
    m_serial->setDataBits(p.dataBits);
    m_serial->setParity(p.parity);
    m_serial->setStopBits(p.stopBits);
    m_serial->setFlowControl(p.flowControl);
    // Запускаем таймер на 100 мс
    m_connectionTimer->start(100);
    if (m_serial->open(QIODevice::ReadWrite)) {
        m_connectionTimer->stop();  // Останавливаем таймер, если соединение успешно
        m_textEdit->setReadOnly(false); // Разрешаем ввод текста
        m_ui->actionConnect->setEnabled(false);
        m_ui->actionDisconnect->setEnabled(true);
        showStatusMessage(tr("Connected to %1 : %2, %3, %4, %5, %6")
                              .arg(p.name).arg(p.stringBaudRate).arg(p.stringDataBits)
                              .arg(p.stringParity).arg(p.stringStopBits).arg(p.stringFlowControl));
    } else {
        m_connectionTimer->stop();  // Останавливаем таймер, если произошла ошибка
        QMessageBox::critical(this, tr("Error"), m_serial->errorString());
        showStatusMessage(tr("Open error"));
    }
}
void MainWindow::closeSerialPort()
{
    if (m_serial->isOpen())
        m_serial->close();
    m_textEdit->setReadOnly(true); // Запрещаем ввод текста, но оставляем виджет активным
    m_ui->actionConnect->setEnabled(true);
    m_ui->actionDisconnect->setEnabled(false);
    m_ui->actionConfigure->setEnabled(true);
    showStatusMessage(tr("Disconnected"));
}
void MainWindow::about()
{
    m_about->show();
}

void MainWindow::writeData(const QByteArray &data)
{
    m_serial->write(data);
}

void MainWindow::readData()
{
    const QByteArray data = m_serial->readAll();
    if (HEX) {
        QString hexData = data.toHex(' ').toUpper();   // Преобразуем данные в HEX формат
        m_textEdit->insertPlainText(hexData + "\r\n"); // Добавляем пробел между байтами для удобства чтения
    } else {
        m_textEdit->insertPlainText(QString::fromUtf8(data)); // Обычный текст
    }
    m_textEdit->ensureCursorVisible(); // Обеспечиваем видимость курсора внизу
    trimBuffer(); // Ограничиваем размер буфера
}


void MainWindow::handleError(QSerialPort::SerialPortError error)
{
    if (error == QSerialPort::ResourceError) {
        QMessageBox::critical(this, tr("Critical Error"), m_serial->errorString());
        closeSerialPort();
    }
}

void MainWindow::showStatusMessage(const QString &message)
{
    m_status->setText(message);
}

void MainWindow::on_actionClear_triggered()
{
    m_textEdit->clear();
}

void MainWindow::on_HEXmode_triggered()
{
    HEX = !HEX; // Переключаем значение флага HEX
}

void MainWindow::trimBuffer()
{
    if (m_textEdit->toPlainText().length() > MaxBufferSize) {
        QTextCursor cursor = m_textEdit->textCursor();
        cursor.movePosition(QTextCursor::Start);
        cursor.select(QTextCursor::LineUnderCursor);
        cursor.removeSelectedText();
        cursor.deleteChar(); // Удаляем первую строку
    }
}



